# reads xbuttler config and return a nice hash
package XButtler::LoadConfig;

use strict;
use XButtler::Common;
use Data::Dumper;
use XML::Simple;
use IO::File;

use vars qw(@EXPORT @ISA);
require Exporter;

@ISA = qw(Exporter);
@EXPORT = qw (LoadConfig);


use constant DEFAULT_DPATH	=> "./";
use constant DEFAULT_NICK	=> "RANDOM";
use constant DEFAULT_USERNAME	=> "buttler";
use constant DEFAULT_IRCNAME	=> "buttler";
use constant DEFAULT_VERSION	=> "The Buttler Effect";
use constant DEFAULT_UMODE	=> "+i";
use constant DEFAULT_DCCALLOW	=> FALSE;  # some bs thats on criten

# dump the raw config that is read in from XML::Simple;
my $dump = FALSE;	
my $debug = FALSE;


# pass in a xbuttler xml config file returns a clean hash
sub LoadConfig {
	my (%args) = @_;

	my $dir = $args{'ConfigDir'};


	my $config;

	if($args{'Debug'} >= 1) {
		$debug = TRUE;
	}

	if($args{'Dump'} >= 1) {
		$dump = TRUE;
	}

	if(!-d $dir) {
		print "LoadConfig: $dir is not a directory\n";
		return undef;
	}

	
	my $defaults = _load_defaults($dir . "/defaults.xml");
	# some error loading defaults
	if(!defined($defaults)) {
		return undef;
	}

	unless(opendir(DIR,$dir)) {
		print "LoadConfig: unable to opendir $dir\n";
		return undef;
	}

	my @contents = readdir(DIR);
	closedir(DIR);

	for my $file (@contents) {
		if(!-d $dir . "/" . $file && $file =~ /^network.*\.xml$/i) {
			my $network = _load_network($dir . "/" . $file,$defaults);
			if(!defined($network)) {
				print "LoadConfig: error loading network file '$dir/$file'\n";
				return undef;
			}
			if(defined($config->{'NETWORKS'}->{$network->{'NAME'}})) {
				print "LoadConfig: error multiple networks have the same <name>$network->{'NAME'}</name> tag.\n";
				return undef;
			} else {
				$config->{'NETWORKS'}->{$network->{'NAME'}} = $network;
			}
		}
	}
	return $config;
}



sub _load_defaults {
	my ($file) = @_;

	my $raw;
	my $clean;


	# file doesnt exist, use build in defaults
	if(!-e $file) {
		$clean->{'DOWNLOAD_PATH'} = DEFAULT_DPATH;
		$clean->{'NICK'} = DEFAULT_NICK;
		$clean->{'IRCNAME'} = DEFAULT_IRCNAME;
		$clean->{'VERSION'} = DEFAULT_VERSION;
		$clean->{'UMODE'} = DEFAULT_UMODE;
		$clean->{'USERNAME'} = DEFAULT_USERNAME;
		$clean->{'DCCALLOW'} = DEFAULT_DCCALLOW;
		return $clean;
	}

	$debug && print trace(), "Loading $file.\n";

	my $fh = IO::File->new($file);
	if($fh) {

		eval {
			$raw = XMLin($fh, keyattr => []);	
		};

		if($@) {
			print "LoadConfig xml error in $file: $@\n";
			return undef;
		}
		$fh->close();

	} else {
		print "LoadConfig unable to open $file for reading\n";
		return undef;
	}


	# dump raw if reqested
	$dump && print Dumper($raw);

	# set defaults
	my $defaults = $raw->{'defaults'};
	if(defined($defaults)) {
		$clean->{'DOWNLOAD_PATH'} = ($defaults->{'dpath'} ? 
			_to_string($defaults->{'dpath'}) : DEFAULT_DPATH);

		if(!-d $clean->{'DOWNLOAD_PATH'}) {
			print "LoadConfig <dpath>$clean->{'DOWNLOAD_PATH'}</dpath> is not a directory in $file\n";
			return undef;
		}

		$clean->{'NICK'} = ($defaults->{'nick'} ? 
			_to_string($defaults->{'nick'}) : DEFAULT_NICK);
		$clean->{'USERNAME'} = ($defaults->{'username'} ? 
			_to_string($defaults->{'username'}) : DEFAULT_USERNAME);
		$clean->{'IRCNAME'} = ($defaults->{'ircname'} ? 
			_to_string($defaults->{'ircname'}) : DEFAULT_IRCNAME);
		$clean->{'VERSION'} = ($defaults->{'version'} ? 
			_to_string($defaults->{'version'}) : DEFAULT_VERSION);
		$clean->{'UMODE'} = ($defaults->{'umode'} ? 
			_to_string($defaults->{'umode'}) : DEFAULT_UMODE);

		if($defaults->{'dccallow'} ne "1" && $defaults->{'dccallow'} ne "0") {
			$clean->{'DCCALLOW'} = DEFAULT_DCCALLOW;
		} else {
			$clean->{'DCCALLOW'} = $defaults->{'dccallow'};
		}
			
	} else {
		$clean->{'DOWNLOAD_PATH'} = DEFAULT_DPATH;
		$clean->{'NICK'} = DEFAULT_NICK;
		$clean->{'IRCNAME'} = DEFAULT_IRCNAME;
		$clean->{'VERSION'} = DEFAULT_VERSION;
		$clean->{'UMODE'} = DEFAULT_UMODE;
	}

	return $clean;
}


sub _load_network {
	my ($file,$defaults) = @_;


	my $raw;
	my $clean;

	$debug && print trace(), "Loading $file.\n";

	my $fh = IO::File->new($file);
	if($fh) {

		eval {
			$raw = XMLin($fh, keyattr => []);	
		};

		if($@) {
			print "LoadConfig xml error in $file: $@\n";
			return undef;
		}
		$fh->close();

	} else {
		print "LoadConfig unable to open $file for reading\n";
		return undef;
	}

	# dump raw if reqested
	$dump && print Dumper($raw);

	my $network = $raw->{'network'};

	if(ref($network) eq "ARRAY") {
		print "LoadConfig only one network defination per file is allowd at this time.\n";
		return undef;
	}

	if(defined($network)) {
		if(defined($network->{'name'})) {
			$clean->{'NAME'} = _to_string($network->{'name'});
		} else {
			print "LoadConfig <name></name> tag required in network defination.\n";
			return undef;
		}

		if(defined($network->{'server'})) {
			$clean->{'SERVERS'} = _to_array($network->{'server'});
		} else {
			print "LoadConfig atleast one server must be defined in network defination.\n";
			return undef;
		}

		
		if(defined($network->{'channel'})) {
			my $channels = _to_array($network->{'channel'});
			for my $channel (@{$channels}) {
				my $cname;
				if(defined($channel->{'name'})) {
					$clean->{'CHANNELS'}->{$channel->{'name'}}->{'NAME'} = $channel->{'name'};
				} else {
					print "LoadConfig <channel> tag requires a name in $file.\n";
				}

				$clean->{'CHANNELS'}->{$channel->{'name'}}->{'KEY'} = _to_string($channel->{'key'});

			}

		} else {
			print "LoadConfig atleast one channel must be defined in network defination.\n";
			return undef;
		}

		$clean->{'NICK'} = ($network->{'nick'} ? 
			_to_string($network->{'nick'}) : $defaults->{'NICK'});

		$clean->{'USERNAME'} = ($network->{'username'} ? 
			_to_string($network->{'username'}) : $defaults->{'USERNAME'});

		$clean->{'IRCNAME'} = ($network->{'ircname'} ? 
			_to_string($network->{'ircname'}) : $defaults->{'IRCNAME'});

		$clean->{'VERSION'} = ($network->{'version'} ? 
			_to_string($network->{'version'}) : $defaults->{'VERSION'});

		$clean->{'DOWNLOAD_PATH'} = ($network->{'dpath'} ? 
			_to_string($network->{'dpath'}) : $defaults->{'DOWNLOAD_PATH'});

		if(!-d $clean->{'DOWNLOAD_PATH'}) {
			print "LoadConfig <dpath>$clean->{'DOWNLOAD_PATH'}</dpath> is not a directory in $file\n";
			return undef;
		}

		$clean->{'UMODE'} = ($network->{'umode'} ? 
			_to_string($network->{'umode'}) : $defaults->{'UMODE'});

		if($network->{'dccallow'} ne "1" && $network->{'dccallow'} ne "0") {
			$clean->{'DCCALLOW'} = $defaults->{'DCCALLOW'};
		} else {
			$clean->{'DCCALLOW'} = $network->{'dccallow'};
		}
	
	} else {
		print "LoadConfig no network defined in $file.\n";
		return undef;
	}		

	return $clean;
	
}

# if the passed object is an ARRAY will return the first element
sub _to_string {
	my ($var) = @_;

	if(!defined($var)) {
		return undef;
	}

	if(ref($var) eq "ARRAY") {
		return @{$var}[0];
	} else {
		return $var;
	}
}

# if the passed object is a string, returns it as an array;
sub _to_array {
                
	my ($var) = @_;
	
	if(!defined($var)) {
		return undef;
	}

	if(ref($var) eq "ARRAY") {
		return $var;
	} else {
		my @tmp;
		push @tmp, $var;  
		return \@tmp;
	} 
}        

1;
